// Program przesyla dane z przetwornika AC do pamieci RAM kanalem DMA.
//
// Do wejscia przetwornika AC podlaczony jest wewnetrzny (w strukturze mikrokontrolera) czujnik temperatury.
//
// Zasada dzialania programu:
//
// 1. Cykliczne przepelnienie timera TIM3 generuje sygnal TRGO - inicjujacy konwersje AC.
// 2. Po zakonczonej konwersji przetwornik AC zglasza zadanie (request) transferu danych kanalem DMA.
// 3. Uklad DMA przesyla dane z Data Register (ADC1->DR) przetwornika AC do pierwszego (i kolejnych)
//    elementu(w) tablicy "buf" (w pamieci RAM).
//
// Zadania.
//
// 1. Skonfigurowac uklad DMA. Ustawic adresy odczytu i zapisu danych, uruchomic program w trybie
//    pojedynczych transferow sampli z przetwornika ADC1 do kolejnych elementow buf[i] (i = 0 ... 7),
//		(jedna konwersja -> jedna probka -> jeden transfer do pamieci pod odpowiednio inkrementowany adres).
//
//		-> Wyswietlic 8 kolejnych odczytow temperatury. Zaobserowac zachowanie ukladu.
//
// 2. Wlaczyc i ustawic kolejke FIFO w ukladzie DMA i wykorzystac transfery seriami
//    (zgromadzenie np. 8 sampli 12(16) bitowych w buforze FIFO -> jeden transfer calosci do pamieci).
//
//		-> Wyswietlic wszystkie temperatury - analogicznie jak w poprzednim punkcie. Co sie zmienilo?
//
// 3. Obliczyc srednia arytmetyczna z 8 ostatnich odczytow i ja wypisac -> sprawdzic wartosc.
//
// 4. - Zmierzyc rzeczwista wartosc napiecia odniesienia Vref przetwornika ADC1 (domyslnie Vref = 3.0 V).
//			Skorygowac skalowanie temperatury, nastepnie sprawdzic wplyw zmiany Vref na poprawnosc pomiarow.

#include <stdio.h>
#include "stm32f4xx.h"
#include "GLCD.h"

#define __FI       1                        // Font index 16x24
#if (__FI == 1)                             // Font index  6x8
  #define __FONT_WIDTH  16
  #define __FONT_HEIGHT 24
#else                                       // Font index 16x24
  #define __FONT_WIDTH   6
  #define __FONT_HEIGHT  8
#endif

// --- 4 ---
// Nalezy sprawdzic i zmierzyc Vref kazdego egzemplarza zestawu Discovery!

#define	Vref 3.0f
#define scale Vref/4096.0f

float t;
uint32_t i;
uint16_t buf[8];
char str[20];

int main(void){

	GLCD_Initialize();
	GLCD_Clear(Black);                        
	GLCD_SetBackColor(Black);
  GLCD_SetTextColor(White);
	
  RCC->AHB1ENR |= (1<<22); // Wlacz "zegar" DMA2.
	RCC->APB1ENR |= (1<<1);  // Wlacz "zegar" TIM3.
	RCC->APB2ENR |= (1<<8);  // Wlacz "zegar" ADC1.

// --- 1 ---

// Konfiguracja DMA (rozdzial 10.5 w Reference Manual, ew. Application Note AN4031).

// Konfiguracja parametrow transferu danych:
	
// 32-bitowy adres urzadzenia peryferyjnego (Peripheral Address Reg.):

	DMA2_Stream0->PAR =
	
// 32-bitowy adres miejsca w pamieci RAM do ktorego beda zapisywane dane (Memory Address Reg.):	

	DMA2_Stream0->M0AR =
	
// Liczba przesylanych danych (probek) w jednej sekwencji:

	DMA2_Stream0->NDTR =

// Opis rejestru SxCR - rozdzial 10.5.5 w RM. W szczegolnosci nalezy zwrocic uwage na: 
// kierunek przeplywu danych,
// typy/rozmiar danych, inkrementacje wskaznikow,
// wlaczyc tryb pierscieniowy (circular)!

	DMA2_Stream0->CR =
	
// --- 2 ---

// Ustawic pelna pojemnosc i wlaczyc bufor FIFO,
// wylaczyc Direct Mode.
// 0pis rejestru SxFCR - rozdzial 10.5.10 w RM.

//	DMA2_Stream0->FCR =
	
	DMA2_Stream0->CR |= 1;			// Wlacz DMA.
	
// Konfiguracja ADC - (zrobiona).

//ADC1->CR1= 0 << 24;													// Rozdzielczosc - 12 bitow.	
	ADC1->CR2 = (1<<28)|(8<<24)|(1<<9)|(1<<8);	// External Tim3 TRGO Trigger/wlacz DMA.
	ADC1->SQR1 = 1<<20;													// Pojedyncza konwersja AC.
	ADC1->SQR3 = 18; 														// Kanal 18.	
	ADC->CCR = (1<<23)|(3<<16);									// Wlacz Temp_Sensor; ADC prescaler = /8.
	ADC1->SMPR1 |= 7<<24;												// Sampling time - maks. - 480 taktow.
	ADC1->CR2 |= 1;															// Wlacz ADC1.

// Konfiguracja timera TIM3. Czestotliwosc probkowania 1 Hz:

	TIM3->PSC = 15999;
	TIM3->ARR = 999;
	TIM3->CR2 |= 2<<4;													// W chwili przepelnienia -> TRGO.
	TIM3->CR1 |= 1;															// Wlacz timer.

	for(;;){
			for (i=0; i<8; i++){
				
// Odczyt wew. czujnika temperatury i obliczenie jej wartosci.
// Surowe sample z ADC sa zapisane przez uklad DMA w buf[i].
// Rozdzial 13.10 Reference Manual, parametry w Datasheet 6.3.22.

// --- 3 ---
// Dopisac: usrednianie ostatnich 8 odczytow temperatury.
 				
				t=(( (scale * (float)buf[i]) - 0.76f)  * 400.0f ) + 25.0f;
				sprintf(str,"t[%u] = %2.2f %cC     ",i,t,0x82);
				GLCD_DisplayString(i,0,__FI,str);
			}
	}
}
